'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (_ref) {
  var
  // Mising "types"
  //  - Comment
  //  - BlockComment
  //  - &attributes
  //  - InterpolatedTag
  //  - Code(buffer: false + block)
  //  - Each(object)
  //  - Mixin
  babel = _ref.babel;
  var parse = _ref.parse;
  var helpers = _ref.helpers;
  var ast = _ref.ast;
  var path = _ref.path;
  var t = babel.types;

  var compiler = {
    unwrap: function unwrap(node) {
      if (t.isJSXExpressionContainer(node)) {
        return node.expression;
      } else {
        return node;
      }
    },
    unwrapExpressionToStatements: function unwrapExpressionToStatements(node) {
      if (t.isDoExpression(node)) {
        return node.body.body;
      } else if (t.isJSXExpressionContainer(node)) {
        return this.unwrapExpressionToStatements(node.expression);
      } else {
        return [t.expressionStatement(node)];
      }
    },
    parseExpression: function parseExpression(src) {
      var val = parse('x = (' + src + ');').program.body;
      (0, _assert2.default)(val.length === 1);
      return val[0].expression.right;
    },
    parseStatement: function parseStatement(src) {
      var val = parse(src).program.body;
      (0, _assert2.default)(val.length === 1);
      return val[0];
    },
    visit: function visit(node) {
      if (typeof this['visit' + node.type] === 'function') {
        return this['visit' + node.type](node);
      } else {
        throw new Error(node.type + ' is not yet supported');
      }
    },
    visitBlock: function visitBlock(node) {
      var _this = this;

      return node.nodes.map(function (node) {
        return _this.visit(node);
      }).filter(Boolean);
    },
    visitCase: function visitCase(node) {
      // compile the case/when into a series of conditionals then compile that down to jsx
      var base = { alternate: null };
      var parent = base;
      var defaultValue = { type: 'Block', nodes: [] };
      node.block.nodes.forEach(function (whenNode) {
        if (whenNode.expr === 'default') {
          defaultValue = whenNode.block;
        } else {
          parent.alternate = {
            type: 'Conditional',
            test: node.expr + ' === ' + whenNode.expr,
            consequent: whenNode.block,
            alternate: null
          };
          parent = parent.alternate;
        }
      });
      if (!base.alternate) {
        throw new Error('Empty case blocks are not allowed');
      }
      parent.alternate = defaultValue;
      return this.visit(base.alternate);
    },
    visitCode: function visitCode(node) {
      if (node.buffer) {
        if (!node.mustEscape) {
          throw new Error('Unescaped attributes are not supported in react-pug');
        }
        return t.jSXExpressionContainer(this.parseExpression(node.val));
      } else {
        // TODO: hoist and rename `const` and `let` variables
        return t.jSXExpressionContainer(t.doExpression(t.blockStatement([this.parseStatement(node.val), t.expressionStatement(t.nullLiteral())])));
      }
    },
    simplifyBlock: function simplifyBlock(nodes) {
      var id = path.scope.generateUidIdentifier('SimpleBlock').name;
      nodes = nodes.map(this.unwrap);
      nodes.forEach(function (node, i) {
        if (t.isJSXElement(node) && !node.openingElement.attributes.some(function (attr) {
          return t.isJSXAttribute(attr) && t.isJSXIdentifier(attr.name, { name: 'key' });
        })) {
          node.openingElement.attributes.push(t.jSXAttribute(t.jSXIdentifier('key'), t.stringLiteral(id + '_' + i)));
        }
      });
      return t.arrayExpression(nodes);
    },
    visitConditional: function visitConditional(node) {
      var test = this.parseExpression(node.test);
      var consequent = this.simplifyBlock(this.visitBlock(node.consequent));
      var alternate = !node.alternate ? t.nullLiteral() : node.alternate.type === 'Block' ? this.simplifyBlock(this.visitBlock(node.alternate)) : this.unwrap(this.visitConditional(node.alternate));
      return t.jSXExpressionContainer(t.conditionalExpression(test, consequent, alternate));
    },
    visitEach: function visitEach(node) {
      var arr = this.parseExpression(node.obj);
      var params = [this.parseExpression(node.val)];
      if (node.key) params.push(this.parseExpression(node.key));
      var body = this.visitBlock(node.block);
      var key = null;
      body.forEach(function (node, i) {
        if (t.isJSXElement(node) && !node.openingElement.attributes.some(function (attr) {
          if (t.isJSXAttribute(attr) && t.isJSXIdentifier(attr.name, { name: 'key' })) {
            if (t.isJSXExpressionContainer(attr.value)) {
              key = attr.value.expression;
            }
            return true;
          }
          return false;
        }) && key) {
          node.openingElement.attributes.push(t.jSXAttribute(t.jSXIdentifier('key'), t.jSXExpressionContainer(t.binaryExpression('+', key, t.stringLiteral('_' + i)))));
        }
      });

      var mapCall = t.callExpression(t.memberExpression(arr, t.identifier('map')), [t.functionExpression(null, params, t.blockStatement([t.returnStatement(body.length === 1 ? body[0] : t.arrayExpression(body))])), t.thisExpression()]);
      var loop = t.jSXExpressionContainer(t.conditionalExpression(t.logicalExpression('&&', arr, t.memberExpression(arr, t.identifier('length'))), body.length === 1 ? mapCall : t.callExpression(helpers.flatten, [mapCall]), node.alternate ? this.simplifyBlock(this.visitBlock(node.alternate)) : t.nullLiteral()));
      return loop;
    },
    visitTag: function visitTag(node) {
      var _this2 = this;

      var name = t.jSXIdentifier(node.name);
      var children = (node.code ? [this.visitCode(node.code)] : []).concat(this.visitBlock(node.block)).reduce(function (a, b) {
        return a.concat(Array.isArray(b) ? b : [b]);
      }, []);

      if (node.attributeBlocks.length) {
        throw new Error('Attribute blocks are not yet supported in react-pug');
      }

      var attrs = node.attrs.map(function (_ref2) {
        var name = _ref2.name;
        var val = _ref2.val;
        var mustEscape = _ref2.mustEscape;

        if (!mustEscape) {
          throw new Error('Unescaped attributes are not supported in react-pug');
        }
        if (/\.\.\./.test(name) && val === true) {
          return t.jSXSpreadAttribute(_this2.parseExpression(name.substr(3)));
        }
        var expr = _this2.parseExpression(val === true ? 'true' : val);
        var jsxValue = t.isStringLiteral(expr) || t.isJSXElement(expr) ? expr : t.jSXExpressionContainer(expr);
        if (/\.\.\./.test(name)) {
          throw new Error('spread attributes must not have a value');
        }
        return t.jSXAttribute(t.jSXIdentifier(name), jsxValue);
      });
      var open = t.jSXOpeningElement(name, attrs, // Array<JSXAttribute | JSXSpreadAttribute>
      children.length === 0);
      var close = children.length === 0 ? null : t.jSXClosingElement(name);
      return t.jSXElement(open, close, children, // ["StringLiteral","JSXExpressionContainer","JSXElement"]
      children.length === 0);
    },
    visitText: function visitText(node) {
      return t.stringLiteral(node.val);
    },
    visitWhile: function visitWhile(node) {
      var _this3 = this;

      var nodes = path.scope.generateUidIdentifier('WhileBlock');
      var statements = [];
      statements.push(t.variableDeclaration('var', [t.variableDeclarator(nodes, t.arrayExpression([]))]));
      var block = this.visitBlock(node.block).map(function (exp) {
        return _this3.unwrapExpressionToStatements(exp);
      });
      var body = [];
      var key = null;
      block.forEach(function (statements, i) {
        var last = statements.pop();
        statements.forEach(function (s) {
          return body.push(s);
        });
        t.assertExpressionStatement(last);
        if (!t.isNullLiteral(last.expression)) {
          if (t.isJSXElement(last.expression) && !last.expression.openingElement.attributes.some(function (attr) {
            if (t.isJSXAttribute(attr) && t.isJSXIdentifier(attr.name, { name: 'key' })) {
              if (t.isJSXExpressionContainer(attr.value)) {
                key = attr.value.expression;
              }
              return true;
            }
            return false;
          }) && key) {
            last.expression.openingElement.attributes.push(t.jSXAttribute(t.jSXIdentifier('key'), t.jSXExpressionContainer(t.binaryExpression('+', key, t.stringLiteral('_' + i)))));
          }
          body.push(t.expressionStatement(t.callExpression(t.memberExpression(nodes, t.identifier('push')), [last.expression])));
        }
      });
      var test = this.parseExpression(node.test);
      statements.push(t.whileStatement(test, t.blockStatement(body)));
      statements.push(t.expressionStatement(nodes));
      return t.jSXExpressionContainer(t.doExpression(t.blockStatement(statements)));
    }
  };
  return compiler.visit(ast);
};

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }